//////////////////////////////////////////////////////////////////////////
// BonSDK .NET binding interface library
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Globalization;

namespace Bon.IEPG
{
	/// <summary>
	/// iEPGf[^̃ACeNXłB
	/// </summary>
	[Serializable]
	internal sealed class IEPGItem : IIEPGItem
	{
		private static readonly char[] alphabets_;

		private readonly Dictionary<string, string> values_;
		private readonly string body_;
		private readonly Dictionary<string, string> stations_;

		static IEPGItem()
		{
			List<char> alphabets = new List<char>();
			for (char a = 'A'; a <= 'Z'; a++)
			{
				alphabets.Add(a);
			}
			for (char a = 'a'; a <= 'z'; a++)
			{
				alphabets.Add(a);
			}
			alphabets_ = alphabets.ToArray();
		}

		public IEPGItem(Dictionary<string, string> values, string body, Dictionary<string, string> stations)
		{
			values_ = values;
			body_ = body;
			stations_ = stations;
		}

		private string SafeGetStation()
		{
			string station = values_["station"];
			int position = station.LastIndexOfAny(alphabets_);
			if (position == -1)
			{
				return stations_[station];
			}
			try
			{
				Enum.Parse(typeof(BroadcastType), station.Substring(position + 1), true);
				int.Parse(station.Substring(position + 1), CultureInfo.InvariantCulture);
			}
			catch (ArgumentException)
			{
				return stations_[station];
			}
			return station;
		}

		public BroadcastType BroadcastType
		{
			get
			{
				string station = SafeGetStation();
				int position = station.LastIndexOfAny(alphabets_);
				if (position == -1)
				{
					throw new FormatException("Invalid station attribute: " + station);
				}
				station = station.Substring(0, position + 1);
				return (BroadcastType)Enum.Parse(typeof(BroadcastType), station, true);
			}
		}

		public int ServiceID
		{
			get
			{
				string station = SafeGetStation();
				int position = station.LastIndexOfAny(alphabets_);
				if (position == -1)
				{
					throw new FormatException("Invalid station attribute: " + station);
				}
				return int.Parse(station.Substring(position + 1), CultureInfo.InvariantCulture);
			}
		}

		public string TransportStreamName
		{
			get
			{
				string stationName = null;
				if (values_.TryGetValue("station-name", out stationName) == true)
				{
					return stationName;
				}
				return values_["station"];
			}
		}

		private DateTime GetBaseDate()
		{
			int year = int.Parse(values_["year"], CultureInfo.InvariantCulture);
			string monthString = values_["month"];
			int month = 1;
			switch (monthString.ToLower())
			{
				case "jan":
					month = 1;
					break;
				case "feb":
					month = 2;
					break;
				case "mar":
					month = 3;
					break;
				case "apr":
					month = 4;
					break;
				case "may":
					month = 5;
					break;
				case "jun":
					month = 6;
					break;
				case "jul":
					month = 7;
					break;
				case "aug":
					month = 8;
					break;
				case "sep":
					month = 9;
					break;
				case "oct":
					month = 10;
					break;
				case "nov":
					month = 11;
					break;
				case "dec":
					month = 12;
					break;
				default:
					month = int.Parse(monthString, CultureInfo.InvariantCulture);
					break;
			}
			int day = int.Parse(values_["date"], CultureInfo.InvariantCulture);
			return new DateTime(year, month, day);
		}

		private TimeSpan ToTimeSpan(string time)
		{
			int position = time.IndexOf(':');
			int hour = int.Parse(time.Substring(0, position), CultureInfo.InvariantCulture);
			int minute = int.Parse(time.Substring(position + 1), CultureInfo.InvariantCulture);
			return new TimeSpan(hour, minute, 0);
		}

		public DateTime StartTime
		{
			get
			{
				DateTime date = GetBaseDate();
				TimeSpan start = ToTimeSpan(values_["start"]);
				return date + start;
			}
		}

		public DateTime EndTime
		{
			get
			{
				DateTime date = GetBaseDate();
				TimeSpan start = ToTimeSpan(values_["start"]);
				TimeSpan end = ToTimeSpan(values_["end"]);
				if (start > end)
				{
					return date.AddDays(1) + end;
				}
				else
				{
					return date + end;
				}
			}
		}

		public string Title
		{
			get
			{
				return values_["program-title"];
			}
		}

		public int ProgramID
		{
			get
			{
				string programIDString = null;
				if (values_.TryGetValue("program-id", out programIDString) == false)
				{
					return -1;
				}
				return int.Parse(programIDString, CultureInfo.InvariantCulture);
			}
		}

		public string Body
		{
			get
			{
				return body_;
			}
		}

		public string this[string name]
		{
			get
			{
				string value = null;
				if (values_.TryGetValue(name, out value) == true)
				{
					return value;
				}
				return null;
			}
		}
	}
}
