//////////////////////////////////////////////////////////////////////////
// BonSDK Stream service
// Copyright (c) http://2sen.dip.jp/dtv/
//
// This is part of BonSDK.
// BonSDK comes with ABSOLUTELY NO WARRANTY;
// for details see 'GNU GENERAL PUBLIC LICENSE 2.0'.

using System;
using System.Collections.Generic;
using System.Net.Sockets;
using System.Runtime.ConstrainedExecution;
using System.Runtime.InteropServices;
using System.Threading;
using NetFwTypeLib;

namespace Bon
{
	internal sealed class FirewallManager : IDisposable
	{
		private static readonly Type firewallManagerType_ = Type.GetTypeFromProgID("HNetCfg.FwMgr", false);
		private static readonly Type openPortType_ = Type.GetTypeFromProgID("HNetCfg.FWOpenPort", false);

		private readonly string policyName_;
		private readonly bool scopeFree_;
		private readonly List<PortInformation> openPorts_ = new List<PortInformation>();
		private Thread thread_;

		public FirewallManager(string policyName, bool scopeFree)
		{
			policyName_ = policyName;
			scopeFree_ = scopeFree;
		}

		~FirewallManager()
		{
			Dispose();
		}

		public void Dispose()
		{
			GC.SuppressFinalize(this);
			if (thread_ != null)
			{
				thread_.Abort();
				thread_.Join();
				thread_ = null;

				INetFwMgr firewallManager = (INetFwMgr)Activator.CreateInstance(firewallManagerType_);
				INetFwProfile profile = firewallManager.LocalPolicy.CurrentProfile;

				lock (this)
				{
					foreach (PortInformation portInformation in openPorts_)
					{
						try
						{
							profile.GloballyOpenPorts.Remove(portInformation.Port, portInformation.Protocol);
						}
						catch (COMException)
						{
						}
					}
				}
			}
		}

		public static bool HasWindowsFirewall
		{
			get
			{
				return (firewallManagerType_ != null) && (openPortType_ != null);
			}
		}

		public void Add(string name, ProtocolType protocol, int port)
		{
			lock (this)
			{
				NET_FW_IP_PROTOCOL_ rawProtocol;
				switch (protocol)
				{
					case ProtocolType.Tcp:
						rawProtocol = NET_FW_IP_PROTOCOL_.NET_FW_IP_PROTOCOL_TCP;
						break;
					case ProtocolType.Udp:
						rawProtocol = NET_FW_IP_PROTOCOL_.NET_FW_IP_PROTOCOL_UDP;
						break;
					default:
						throw new ArgumentException("Cannot filter protocol: " + protocol.ToString());
				}

				string rawName = string.Format("{0} ({1})", policyName_, name);

				PortInformation portInformation = new PortInformation(rawName, rawProtocol, port);
				openPorts_.Add(portInformation);
			}
		}

		public void Commit()
		{
			thread_ = new Thread(new ThreadStart(this.OpenPortEntry));
			thread_.SetApartmentState(ApartmentState.MTA);
			thread_.Start();
		}

		private void OpenPortEntry()
		{
			try
			{
				INetFwMgr firewallManager = (INetFwMgr)Activator.CreateInstance(firewallManagerType_);
				int retry = 50;
				while (true)
				{
					try
					{
						INetFwProfile profile = firewallManager.LocalPolicy.CurrentProfile;

						lock (this)
						{
							foreach (PortInformation portInformation in openPorts_)
							{
								bool found = false;
								foreach (INetFwOpenPort openPort in profile.GloballyOpenPorts)
								{
									if (openPort.Name == portInformation.Name)
									{
										openPort.Protocol = portInformation.Protocol;
										openPort.Port = portInformation.Port;
										openPort.Scope = scopeFree_ ? NET_FW_SCOPE_.NET_FW_SCOPE_ALL : NET_FW_SCOPE_.NET_FW_SCOPE_LOCAL_SUBNET;
										openPort.Enabled = true;
										found = true;
										break;
									}
								}
								if (found == false)
								{
									INetFwOpenPort openPort = (INetFwOpenPort)Activator.CreateInstance(openPortType_);
									openPort.Name = portInformation.Name;
									openPort.Protocol = portInformation.Protocol;
									openPort.Port = portInformation.Port;
									openPort.Scope = scopeFree_ ? NET_FW_SCOPE_.NET_FW_SCOPE_ALL : NET_FW_SCOPE_.NET_FW_SCOPE_LOCAL_SUBNET;
									openPort.Enabled = true;
									profile.GloballyOpenPorts.Add(openPort);
								}
							}
						}
						break;
					}
					catch (COMException ex)
					{
						// NƎs邱Ƃ̂ŁAgC
						// iT[rXˑ"SharedAccess"ǉĂʖځj
						if ((uint)Marshal.GetHRForException(ex) != 0x800706d9)
						{
							throw;
						}
						retry--;
						if (retry <= 0)
						{
							throw;
						}
					}
					Thread.Sleep(1000);
				}

				Logger.Write("Open ports with windows firewall.");
			}
			catch (Exception ex)
			{
				Logger.Write(ex);
			}
		}

		private sealed class PortInformation
		{
			private readonly string name_;
			private readonly NET_FW_IP_PROTOCOL_ protocol_;
			private readonly int port_;

			public PortInformation(string name, NET_FW_IP_PROTOCOL_ protocol, int port)
			{
				name_ = name;
				protocol_ = protocol;
				port_ = port;
			}

			public string Name
			{
				get
				{
					return name_;
				}
			}

			public NET_FW_IP_PROTOCOL_ Protocol
			{
				get
				{
					return protocol_;
				}
			}

			public int Port
			{
				get
				{
					return port_;
				}
			}
		}
	}
}
